<?php
if (!defined("SLIDEDECK_HOTSPOTS_DIRNAME")) define("SLIDEDECK_HOTSPOTS_DIRNAME", dirname(__FILE__));
if (!defined("SLIDEDECK_HOTSPOTS_URLPATH")) define("SLIDEDECK_HOTSPOTS_URLPATH", trailingslashit(plugins_url()) . 'slidedeck-addons/' . basename(SLIDEDECK_HOTSPOTS_DIRNAME));
if (!defined("SLIDEDECK_HOTSPOTS_VERSION")) define("SLIDEDECK_HOTSPOTS_VERSION", "1.0.0");

if ( ! class_exists('SlideDeck_Hotspots')) {

    class SlideDeck_Hotspots {

        var $namespace           = "slidedeck";
        var $package_slug        = 'slidedeck-hotspots';
        static $st_friendly_name = "SlideDeck Hotspots";

        public $errors = array();

        function __construct()
        {
            // Fail silently if SlideDeck core is not installed
            if ( ! class_exists('SlideDeckPlugin') ) {
                return false;
            }

            SlideDeckPlugin::$addons_installed[$this->package_slug] = $this->package_slug;

            $this->slidedeck_namespace = SlideDeckPlugin::$st_namespace;

            $disable_hotspot = get_option( 'slidedeck_hotspots_disable_addon', 2 );

            if( $disable_hotspot == 2 ){
                add_action( 'wp_enqueue_scripts', array( $this , 'front_scripts' ) );
            }

            // Add js/css file only on sliddeck slider editor page.
            add_action('admin_enqueue_scripts' , array(&$this, 'admin_print_scripts'));
            add_action('admin_enqueue_scripts', array(&$this, 'admin_print_styles'));

            $this->create_database();

            add_action( "wp_ajax_{$this->namespace}_update_hotspot" , array( $this , 'update_hotspot' ) );
            add_action( "wp_ajax_nopriv_{$this->namespace}_update_hotspot" , array( $this , 'update_hotspot' ) );

            add_action( "wp_ajax_{$this->namespace}_pull_stats" , array( $this , 'pull_stats' ) );


            add_action('admin_menu', array( $this , 'add_admin_pages' ), 99 );

            //call register settings function
            add_action( 'admin_init', array( $this , 'register_settings' ) );

        }


        /**
         * Initialization function to hook into the WordPress init action
         *
         * Instantiates the class on a global variable and sets the class, actions
         * etc. up for use.
         */
        static public function instance() {
            global $SlideDeckHotspots;

            $slidedeck_version = defined('SLIDEDECK_VERSION') ? SLIDEDECK_VERSION : "2.0";

            if (version_compare($slidedeck_version, '1.0', ">=")) {
                // Only instantiate the Class if it hasn't been already
                if (!isset($SlideDeckHotspots)) {
                    $SlideDeckHotspots = new SlideDeck_Hotspots();
                }
            }
        }

        /**
         * Enqueue javascript file.
         */
        public function admin_print_scripts() {

            if( isset( $_GET['page'] ) && $_GET['page'] === 'slidedeck-stats' ){
                wp_enqueue_script($this->package_slug . "-js", SLIDEDECK_HOTSPOTS_URLPATH . '/js/default.js', array("jquery", "jquery-ui-core", "jquery-ui-accordion"));
            }
        }

        public function front_scripts()
        {
            wp_enqueue_script( "jquery-viewport", SLIDEDECK_HOTSPOTS_URLPATH . '/js/vendors/jquery.viewport.mini.js', array("jquery"));
            wp_enqueue_script($this->package_slug . "-js", SLIDEDECK_HOTSPOTS_URLPATH . '/js/hotspot.js', array("jquery", 'jquery-viewport'));

            wp_localize_script( $this->package_slug . "-js", 'slideckVars', array(
                'ajax_url' => admin_url( 'admin-ajax.php' ),
                'unique_pageviews' => get_option( 'slidedeck_hotspots_unique_views' , 1 ) // 1 is yes, 2 is no
            ));
        }

        /**
         * Enqueue css file.
         */
        public function admin_print_styles() {
            if( isset( $_GET['page'] ) && $_GET['page'] === 'slidedeck-stats' ){
                wp_enqueue_style($this->package_slug . "-css", SLIDEDECK_HOTSPOTS_URLPATH . '/css/default.css' , array( 'slidedeck-admin' ) );
            }
        }

        public function get_table_name()
        {
            global $wpdb;

            $table_name = $wpdb->prefix . $this->namespace . "_stats";

            return $table_name;
        }

        public function create_database()
        {
           global $wpdb;

           $table_name = $this->get_table_name();

           $charset_collate = $wpdb->get_charset_collate();

           if( $wpdb->get_var("show tables like '{$table_name}'") != $table_name) {

                $sql = "CREATE TABLE IF NOT EXISTS $table_name (
                  id mediumint(9) NOT NULL AUTO_INCREMENT,
                  ip_address varchar(55) NOT NULL,
                  time datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
                  browser text DEFAULT '' NOT NULL,
                  slide_index int(4) NOT NULL,
                  action text NOT NULL,
                  caption text NOT NULL,
                  post_id int(12) DEFAULT 0 NOT NULL,
                  PRIMARY KEY (id)
                ) $charset_collate;";

                require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

                dbDelta( $sql );
           }

        }

        function get_user_ip()
        {
            if ( ! empty( $_SERVER['HTTP_CLIENT_IP'] ) )
            {
                //check ip from share internet
                $ip = $_SERVER['HTTP_CLIENT_IP'];
            }
            elseif ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) )
            {
                //to check ip is pass from proxy
                $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
            }
            else
            {
                $ip = $_SERVER['REMOTE_ADDR'];
            }

            return $ip;
        }

        function get_browser()
        {
            $u_agent = $_SERVER['HTTP_USER_AGENT'];
                $bname = 'Unknown';
                $platform = 'Unknown';
                $version= "";

                //First get the platform?
                if (preg_match('/linux/i', $u_agent)) {
                    $platform = 'linux';
                }
                elseif (preg_match('/macintosh|mac os x/i', $u_agent)) {
                    $platform = 'mac';
                }
                elseif (preg_match('/windows|win32/i', $u_agent)) {
                    $platform = 'windows';
                }

                // Next get the name of the useragent yes seperately and for good reason
                if(preg_match('/MSIE/i',$u_agent) && !preg_match('/Opera/i',$u_agent))
                {
                    $bname = 'Internet Explorer';
                    $ub = "MSIE";
                }
                elseif(preg_match('/Firefox/i',$u_agent))
                {
                    $bname = 'Mozilla Firefox';
                    $ub = "Firefox";
                }
                elseif(preg_match('/Chrome/i',$u_agent))
                {
                    $bname = 'Google Chrome';
                    $ub = "Chrome";
                }
                elseif(preg_match('/Safari/i',$u_agent))
                {
                    $bname = 'Apple Safari';
                    $ub = "Safari";
                }
                elseif(preg_match('/Opera/i',$u_agent))
                {
                    $bname = 'Opera';
                    $ub = "Opera";
                }
                elseif(preg_match('/Netscape/i',$u_agent))
                {
                    $bname = 'Netscape';
                    $ub = "Netscape";
                }

                // finally get the correct version number
                $known = array('Version', $ub, 'other');
                $pattern = '#(?<browser>' . join('|', $known) .
                ')[/ ]+(?<version>[0-9.|a-zA-Z.]*)#';
                if (!preg_match_all($pattern, $u_agent, $matches)) {
                    // we have no matching number just continue
                }

                // see how many we have
                $i = count($matches['browser']);
                if ($i != 1) {
                    //we will have two since we are not using 'other' argument yet
                    //see if version is before or after the name
                    if (strripos($u_agent,"Version") < strripos($u_agent,$ub)){
                        $version= $matches['version'][0];
                    }
                    else {
                        $version= $matches['version'][1];
                    }
                }
                else {
                    $version= $matches['version'][0];
                }

                // check if we have a number
                if ($version == null || $version == "") {$version = "?";}

                return array(
                    'userAgent' => $u_agent,
                    'name'      => $bname,
                    'version'   => $version,
                    'platform'  => $platform,
                    'pattern'    => $pattern
                );
        }

        public function update_hotspot_table( $index = '' , $post_id, $action, $caption )
        {
            global $wpdb;

            $table_name       = $this->get_table_name();
            $ip_address       = $this->get_user_ip();
            $browser_data     = $this->get_browser();
            $browser          = $browser_data['name'];

            $output = $wpdb->insert(
             $table_name,
             array(
                  'ip_address'  => $ip_address,
                  'time'        => current_time( 'mysql' ),
                  'browser'     => $browser,
                  'slide_index' => $index, //zero based
                  'action'      => $action,
                  'caption'     => $caption,
                  'post_id'     => $post_id,
             )
            );
        }

        public function update_hotspot()
        {

            $slides = isset( $_POST['slides'] ) ? $_POST['slides'] : false;

            $errors = '';

            if( is_array( $slides ) ){
                foreach ($slides as $slide )
                {
                    $index   = isset( $slide[0] ) && $slide[0] ?  intval($slide[0]) : false;
                    $post_id = isset( $slide[1] ) ? intval( $slide[1] ) : false;
                    $action  = isset( $slide[2] ) ? $slide[2] : false;
                    $caption = isset( $slide[3] ) ? $slide[3] : false;

                    if( ! in_array( $action, array( 'view', 'click' ) ) || ! $post_id ){
                        $errors = "Couldnt get in";
                        continue;
                    }

                    $this->update_hotspot_table( $index , $post_id, $action, $caption );
                }
            }

            echo json_encode( array( 'sucsess' => true, 'slides' => $slides, 'error' => $errors ) );

            die();

        }

        /**
         * Renders the html with the given variables
         * @param  [string] $filePath the file path after /view/
         * @param  [array] $viewData
         * @return the markup
         */
        function render( $filePath, $viewData = null ){

            // Was any data sent through?
            ( $viewData ) ? extract( $viewData ) : null;

            ob_start();

            include ( SLIDEDECK_HOTSPOTS_DIRNAME . '/views/' . $filePath . '.php' );
            $template = ob_get_contents();

            ob_end_clean();

            return $template;

        }

        public function add_admin_pages()
        {
            add_submenu_page( 'slidedeck.php', __( 'Lead Generation Stats' , 'slidedeck' ) , __( 'Lead Generation Stats' , 'slidedeck' ) , 'manage_options', 'slidedeck-stats' , array( $this , 'add_stats_page' ) );
        }

        public function register_settings()
        {
            register_setting( 'slidedeck-hotspots-group', 'slidedeck_hotspots_disable_addon' );
            register_setting( 'slidedeck-hotspots-group', 'slidedeck_hotspots_unique_views' );
        }

        public function add_stats_page()
        {

            include_once SLIDEDECK_HOTSPOTS_DIRNAME . '/views/stats-page.php';
        }

        public function render_table( $post_id = false, $count = 0 )
        {

            $error = __( "This slider is not viewed or clicked yet.", 'slidedeck' );

            if( $count == 0 ) return $error;

            global $wpdb;
            $table_name       = $wpdb->prefix . "slidedeck_stats";
            $results_per_page = 100;
            $get_name         = 'resultcount';
            $get_value        = isset( $_GET[$get_name] ) ? intval($_GET[$get_name]) : 1;
            $offset           = ( $get_value * $results_per_page ) - $results_per_page;

            //Get total rows
            // $total_results = $wpdb->get_results( "SELECT COUNT(*) AS total_rows FROM {$table_name}" );
            // $total_rows = isset( $total_results[0] ) ? $total_results[0]->total_rows : 0;

            //Get results
            $view_results = $wpdb->get_results(
                "SELECT caption, slide_index, COUNT( slide_index ) as view_count
                FROM  {$table_name}
                WHERE post_id = {$post_id}
                AND ACTION =  'view'
                GROUP BY slide_index", OBJECT
            );

            $click_results = $wpdb->get_results(
                "SELECT caption, slide_index, COUNT( slide_index ) as click_count
                FROM  {$table_name}
                WHERE post_id = {$post_id}
                AND ACTION =  'click'
                GROUP BY slide_index", OBJECT
            );

            return $this->render( "stats-table" , array( 'views' => $view_results, 'clicks' => $click_results ) );

        }

        public function pull_stats()
        {
            $post_id = isset( $_POST['post_id'] ) ? $_POST['post_id'] : false;

            if( ! $post_id ) return;

            echo json_encode( array( "html" => $this->render_table( $post_id , 1 ) ) ); //1 if it may have value

            die();
        }

    }
}
