<?php
if (!defined("SLIDEDECK_IMPORT_EXPORT_DIRNAME"))
    define("SLIDEDECK_IMPORT_EXPORT_DIRNAME", dirname(__FILE__));
if (!defined("SLIDEDECK_IMPORT_EXPORT_URLPATH"))
    define("SLIDEDECK_IMPORT_EXPORT_URLPATH", trailingslashit(plugins_url()) . 'slidedeck-addons/' . basename(SLIDEDECK_IMPORT_EXPORT_DIRNAME));
if (!defined("SLIDEDECK_IMPORT_EXPORT_VERSION"))
    define("SLIDEDECK_IMPORT_EXPORT_VERSION", "1.0.0");

if (!class_exists('SlideDeck_Import_Export')) {

    class SlideDeck_Import_Export {

        var $namespace = "slidedeck_import_export";
        var $package_slug = 'slidedeck-import-export';
        static $st_friendly_name = "SlideDeck Import/Export";
        public $errors = array();

        function __construct() {
            $slidedeckClass = '';
			// Fail silently if SlideDeck core is not installed
			if ( !class_exists( 'SlideDeckPlugin' ) && !class_exists( 'SlideDeckLitePlugin' )  ) {
				return false;
			}
			if(class_exists( 'SlideDeckPlugin' )){
				$slidedeckClass = 'SlideDeckPlugin';
			}
			else if(class_exists( 'SlideDeckLitePlugin' )){
				$slidedeckClass = 'SlideDeckLitePlugin';
			}
			$slidedeckClass::$addons_installed[$this->package_slug] = $this->package_slug;

			$this->slidedeck_namespace = $slidedeckClass::$st_namespace;


            // Add Import/Export Slider section.
            add_action('slidedeck_after_options_group_wrapper', array(&$this, 'slidedeck_import_export_option'), 999);

            // Add js/css file only on sliddeck slider editor page.
            add_action('admin_print_scripts-toplevel_page_' . SLIDEDECK_HOOK, array(&$this, 'admin_print_scripts'));
            add_action('admin_print_styles-toplevel_page_' . SLIDEDECK_HOOK, array(&$this, 'admin_print_styles'));

            // Export slideck slider into json file.
            add_action('admin_init', array(&$this, 'slidedeck_slider_export'));
            // Import slidedeck slider from json file.
            add_action('admin_init', array(&$this, 'slidedeck_slider_import'));

            // Add any notices generated by the import script.
            add_action('admin_notices', array($this, 'notices'));
        }

        /**
         * Initialization function to hook into the WordPress init action
         *
         * Instantiates the class on a global variable and sets the class, actions
         * etc. up for use.
         */
        static public function instance() {
            global $SlideDeckImportExport;

            $slidedeck_version = defined('SLIDEDECK_VERSION') ? SLIDEDECK_VERSION : "2.0";

            if (version_compare($slidedeck_version, '1.0', ">=")) {
                // Only instantiate the Class if it hasn't been already
                if (!isset($SlideDeckImportExport)) {
                    $SlideDeckImportExport = new SlideDeck_Import_Export();
                }
            }
        }

        /**
         * Enqueue javascript file.
         */
        public function admin_print_scripts() {
            wp_enqueue_script($this->package_slug . "-js", SLIDEDECK_IMPORT_EXPORT_URLPATH . '/js/default.js', array("jquery"));
        }

        /**
         * Enqueue css file.
         */
        public function admin_print_styles() {
            wp_enqueue_style($this->package_slug . "-css", SLIDEDECK_IMPORT_EXPORT_URLPATH . '/css/default.css');
        }

        /**
         * Display import/export option on slider settings page.
         * @global type $SlideDeckPlugin
         * @param type $slidedeck
         */
        public function slidedeck_import_export_option($slidedeck) {
            global $SlideDeckPlugin;

            include( SLIDEDECK_IMPORT_EXPORT_DIRNAME . '/views/_options.php' );
        }

        /**
         * Export slidedeck slider.
         */
        public function slidedeck_slider_export() {
            // check slider export is available.
            if (!isset($_POST['slidedeck-export-submit'])) {
                return;
            }
            // cehck slider export nonce is valid and verified.
            if (!( isset($_POST['slidedeck-export']) && wp_verify_nonce($_POST['slidedeck-export'], 'slidedeck-export') )) {
                return;
            }

            // Ignore the user aborting the action.
            ignore_user_abort(true);

            // Grab the slider id.
            $slider_id = absint($_POST['slidedeck_slider_id']);
            $slider_info = get_post($slider_id);
            $slider_settings = get_post_meta($slider_id);
            
            $slider_data = array(
                "ID" => $slider_id,
                "slider_title" => $slider_info->post_title,
                "slider_type" => $slider_info->post_type,
                "slider_settings" => wp_list_pluck( $slider_settings, 0 ),
            );
            $slider_slides = get_posts(
                    array(
                        "post_parent" => $slider_id,
                        'post_type' => 'sd2_custom_slide',
                        "posts_per_page" => -1,
                        "order" => "ASC",
                        "orderby" => "ID",
                    )
            );

            if (!empty($slider_slides)) {
                foreach ($slider_slides as $slider_slide) {
                    $slide_data = array(
                        "ID" => $slider_slide->ID,
                        "slide_title" => $slider_slide->post_title,
                        "slide_type" => $slider_slide->post_type,
                        "slide_content" => $slider_slide->post_content,
                        "slide_excerpt" => $slider_slide->post_excerpt,
                        "slide_meta" => wp_list_pluck(get_post_meta($slider_slide->ID), 0)
                    );
                    if (isset($slide_data['slide_meta']['_image_attachment']) &&
                            !empty($slide_data['slide_meta']['_image_attachment'])) {
                        $slide_data['_image_attachment_url'] = wp_get_attachment_url($slide_data['slide_meta']['_image_attachment']);
                    }
                    $slider_data['slides'][] = $slide_data;
                }
            } else {
                $slider_data['slides'] = array();
            }

            // Set the proper headers.
            nocache_headers();
            header('Content-Type: application/json; charset=utf-8');
            header('Content-Disposition: attachment; filename=slidedeck-' . $slider_id . '-' . date('d-m-Y') . '.json');
            header('Expires: 0');

            // Make the settings downloadable to a JSON file and die.
            die(json_encode($slider_data));
        }

        /**
         * Export slidedeck slider.
         */
        public function slidedeck_slider_import() {

            // check slider import is available.
            if (!isset($_POST['slidedeck-import-submit'])) {
                return;
            }
            // cehck slider export nonce is valid and verified.
            if (!( isset($_POST['slidedeck-import']) && wp_verify_nonce($_POST['slidedeck-import'], 'slidedeck-import') )) {
                return;
            }
            //check import file.
            if ((!isset($_FILES['slidedeck_import_slider']['name']) && !empty($_FILES['slidedeck_import_slider']['name']) ) ||
                    !isset($_FILES['slidedeck_import_slider']['tmp_name']) && !empty($_FILES['slidedeck_import_slider']['tmp_name'])) {
                $this->errors[] = __('Sorry, but there are no files available to import a slider.', 'slidedeck');
                return;
            }
            //check correct file name.
            if (!preg_match('#^slidedeck#i', $_FILES['slidedeck_import_slider']['name'])) {
                $this->errors[] = __('Sorry, but you have attempted to upload a slider import file with an incompatible filename. Slidedeck import files must begin with "slidedeck".', $this->slidedeck_namespace);
                return;
            }
            //check correct file type.
            $file_array = explode('.', $_FILES['slidedeck_import_slider']['name']);
            $extension = end($file_array);
            if (!( 'json' === $extension )) {
                $this->errors[] = __('Sorry, but Slidedeck import files must be in <code>.json</code> format.', $this->slidedeck_namespace);
                return;
            }

            $file = $_FILES['slidedeck_import_slider']['tmp_name'];
            $contents = @file_get_contents($file);

            if (!$contents) {
                $this->errors[] = __('Sorry, but there was an error retrieving the contents of the slider export file. Please try again.', $this->slidedeck_namespace);
                return;
            }

            $slider_data = json_decode($contents, true);
            $slider_id = absint($_POST['slidedeck_slider_id']);

            /* Step 1 : Import slider information and its settings. Old data will be override. */

            // Prepare data for slider info.
            $exists_post_id = post_exists( $slider_data['slider_title'] );
            
            $update_slider = array(
                "ID" => $slider_id,
                "post_title" => ( $exists_post_id != $slider_id ) ? $slider_data['slider_title']. ' Copy' : $slider_data['slider_title'],
                "post_status" => "publish",
            );
            //Update slider basic info.
            wp_update_post($update_slider);

            //Update slider settings and its meta information.
            foreach ($slider_data['slider_settings'] as $meta_key => $meta_value) {
                update_post_meta($slider_id, $meta_key, maybe_unserialize( $meta_value ) );
            }

            /* Step 2: Import slides in slider if any. Old slide will be removed. */

            if ( isset( $slider_data['slides'] ) && !empty( $slider_data['slides'] ) ) {

                //First remove all old slide of the slider.
                $slider_slides = get_posts(
                        array(
                            "post_parent" => $slider_id,
                            'post_type' => 'sd2_custom_slide',
                            "posts_per_page" => -1,
                            "order" => "ASC",
                            "orderby" => "ID",
                            "post_status" => "any",
                        )
                );

                if (is_array($slider_slides) && count($slider_slides) > 0) {
                    // Delete all the Children of the Parent Page
                    foreach ($slider_slides as $slider_slide) {
                        wp_delete_post($slider_slide->ID, true);
                    }
                }

                //Import each slide and its settings.
                foreach ($slider_data['slides'] as $slide) {
                    $new_slide = array(
                        'post_parent' => $slider_id,
                        'post_title' => $slide['slide_title'],
                        'post_type' => $slide['slide_type'],
                        'post_content' => $slide['slide_content'],
                        'post_excerpt' => $slide['slide_excerpt'],
                        "post_status" => "publish",
                    );
                    // Create new slide.
                    $slide_id = wp_insert_post($new_slide);

                    if (!$slide_id) { //If slide not create then continue process for next slide.
                        continue;
                    }

                    // Add slide settings.
                    foreach ($slide['slide_meta'] as $meta_key => $meta_value) {
                        update_post_meta($slide_id, $meta_key, maybe_unserialize( $meta_value ) );
                    }

                    //If slide image source is upload then download image and create attachment and update slide setting.
                    if ( isset( $slide['_image_attachment_url'] ) && !empty( $slide['_image_attachment_url'] ) ) {
                        $image = wp_get_attachment_image_src( $slide['slide_meta']['_image_attachment'] );

                        if (!$image) {
                            // Get the image from a remote URL.
                            $attachemt_id = $this->import_remote_image( $slide['_image_attachment_url'], $slide_id );
                            
                            // Update slide with new attachement id.
                            if( $attachemt_id ) {
                                update_post_meta( $slide_id, '_image_attachment', $attachemt_id );
                            }
                        }
                    }
                }
            }
            
            wp_redirect( slidedeck_action( "&action=edit&slidedeck=" . $slider_id . "&slidedeck-imported=1"  ) );
        }
        
        /**
         * Get remote image and upload it and return attachement id.
         * @param string $image_url
         * @param int $slide_id
         * @return type int attachement id.
         */
        public function import_remote_image( $image_url, $slide_id ) {
            // Prepare variables.
            $stream = wp_remote_get( $image_url, array( 'timeout' => 60 ) );
            $type = wp_remote_retrieve_header( $stream, 'content-type' );

            // If we cannot get the image or determine the type, skip over the image.
            if (is_wp_error( $stream ) || !$type ) {
                $this->errors[] = __( 'Could not retrieve a valid image from the URL ' . $image_url . '.', $this->slidedeck_namespace );
                return 0;
            } else {
                // It is an image. Stream the image.
                $mirror = wp_upload_bits( basename( $image_url ), null, wp_remote_retrieve_body( $stream ) );

                // If there is an error, bail.
                if ( !empty( $mirror['error'] ) ) {
                    $this->errors[] = $mirror['error'];
                    return 0;
                } else {
                    $attachment = array(
                        'post_title' => basename( $image_url ),
                        'post_mime_type' => $type
                    );
                    $attach_id = wp_insert_attachment( $attachment, $mirror['file'], $slide_id );

                    // Generate and update attachment metadata.
                    if ( !function_exists( 'wp_generate_attachment_metadata' ) ) {
                        require ABSPATH . 'wp-admin/includes/image.php';
                    }

                    // Generate and update attachment metadata.
                    $attach_data = wp_generate_attachment_metadata( $attach_id, $mirror['file'] );
                    wp_update_attachment_metadata( $attach_id, $attach_data );
                    
                    return $attach_id;
                }
            }
        }

        /**
         * Outputs any errors or notices generated by the class.
         */
        public function notices() {

            if (!empty($this->errors)) :
                ?>
                <div id="message" class="error">
                    <p><?php echo implode('<br>', $this->errors); ?></p>
                </div>
                <?php
            endif;

            // If a slider has been imported, create a notice for the import status.
            if ( isset( $_GET['slidedeck-imported'] ) && $_GET['slidedeck-imported'] ) :
                ?>
                <div id="message" class="updated">
                    <p><?php _e( 'Slidedeck slider imported. Please check to ensure all images and data have been imported properly.', $this->slidedeck_namespace ); ?></p>
                </div>
                <?php
            endif;
        }
    }
}
