
/**
 * SlideDeck Lens Admin Scripts
 *
 * This script file will be loaded automatically in the SlideDeck edit view
 * of the WordPress control panel when this lens is chosen.
 *
 * You must include all your JavaScript in the SlideDeckLensAdmin[LENS_SLUG]
 * function, or it will not be run when the Lens is chosen in real time by
 * the user in the editing view.
 */

jQuery(function($){

	if( typeof SDTiled !== 'undefined' ) return;

	window.SDTiled = {

		$currentEditor : false,

		init : function(){
			this.imageUploader();
			this.events();
		},

		events : function(){
			$('body').on( 'click' , '.sdt-cross' , this.removeImage  );
			$('body').on( 'change' , '#sdt-checkbox-just-effect', this.showScrollContent );
			$('body').on( 'change' , '#sdt-checkbox-layers', this.showMouseMoveContent );
			$('body').on( 'click' , '#slidedeck-section-lenses label.lens', this.addLensClasses );
			$('#slidedeck-section-header').on( 'slideEditorOpen' , this.onEditorOpen );
			$('#slidedeck-preview').on('load', this.createSliderLoadedEvent );
			$('#slidedeck-preview').on('tiledSlideisReady', this.onTiledSlideReady );
			$('body').on('click', '#slidedeck-section-options' , this.hideOptions );
			$('body').on( 'click', '#slidedeck-content-control .slide', this.setCurrentEditor );
			$(document).on('ready', this.hideOptions);
			$(window).on('load', this.hideOptions);
		},

		showScrollContent : function(){
			/*
                    $('.sdt-mouse-move-layers ').addClass('sdt-hide');
			$('.sdt-scroll-layers').removeClass('sdt-hide').fadeIn();
			$('.sdt_tiled_type_field').val('scroll');
                        */
		},

		showMouseMoveContent : function(){
			/*
                    $('.sdt-scroll-layers').addClass('sdt-hide');
			$('.sdt-mouse-move-layers ').removeClass('sdt-hide').fadeIn();
			$('.sdt_tiled_type_field').val('mousemove');
                        */
		},

		setCurrentEditor : function(){
			SDTiled.$currentEditor = $(this);
		},

		hideOptions : function(){

			var $mainForm = $('#slidedeck-update-form');

			if( SDTiled.isTiled() ){
				$mainForm.addClass('is-tiled-lens');
			}
			else{
				$mainForm.removeClass('is-tiled-lens');
			}

			SDTiled.removeOtherSlideEditors();
		},

		removeImage : function()
		{
			var $parentLi = $(this).closest('li');
			$parentLi.find('.sdt-attachment-id').val('');
			$parentLi.find('.sdt-url').text('');
			$parentLi.find('.sdt-thumbnail').html( '' );
			$parentLi.removeClass('sdt_has_thumb');
		},

		imageUploader : function(){

			var $this, image, imageData, uploaded_image, $parentLi, attachment_id, thumbnail;

		    $('body').on( 'click' , '.sdt-uploader' , function(e) {
				e.preventDefault();
				$this = $(this);
		        image = wp.media({
		            title: 'Upload Slide Image',
		            // mutiple: true if you want to upload multiple files at once
		            multiple: false
		        }).open()
		        .on('select', function(e){
		            // This will return the selected image from the Media Uploader, the result is an object
		            uploaded_image = image.state().get('selection').first();
		            // We convert uploaded_image to a JSON object to make accessing it easier
		            // Output to the console uploaded_image
		            imageData = uploaded_image.toJSON();

		            if( $.inArray( imageData.subtype , [ 'jpg','png','PNG','JPG','JPEG', 'jpeg', 'gif' ,'GIF' ] ) < 0 ){
		            	alert("Sorry, You can only upload images in this lens.");
		            	return;
		            }

		            thumbnail = imageData.subtype === 'gif' ? imageData.url : imageData.sizes.thumbnail.url;

	            	$parentLi = $this.closest('li');

	            	attachment_id = imageData.id;

		            // Let's assign the url value to the input field
		            $parentLi.find('.sdt-attachment-id').val(attachment_id);
		            $parentLi.find('.sdt-url').text( imageData.name + '.' + imageData.subtype );
		            $parentLi.addClass('sdt_has_thumb');
		            $parentLi.find('.sdt-thumbnail').html( '<img src=" ' + thumbnail + ' " >' );
		        });
		    });
		},

		isTiled : function(){
			var currentLens = $('#slidedeck-section-lenses').find('label.lens.selected').find('span.title').text().trim();
			return currentLens === 'Tiled';
		},

		onEditorOpen : function( e, editor ){

			if( ! SDTiled.isTiled() ) return;

			SDTiled.showMessage( e, editor );

			//Show hide radio content
			var $editor = $(editor);
			var $checkbox = $editor.find('.sdt-slide-checkboxes');

			//Add Class
			$editor.addClass('sdt-slide-editor');

			if( $checkbox.length ){
				if( $checkbox.find("input[type='radio']:checked").val() === 'mousemove' ){
					SDTiled.showMouseMoveContent();
				}
				else{
					SDTiled.showScrollContent();
				}
			}

			$('#slidedeck-choose-slide-type').addClass('sdt-slide-editor');
		},

		showMessage : function( e, editor ){

			if( SDTiled.$currentEditor ){
				if( SDTiled.$currentEditor.hasClass('is-tiled-image-only') ){
					var $header = $(editor).find('.slide-type-header');
					if( $header.length ){
						$header.after("<p class='sdt-image-only-message'>Tiled Lens only works with images. Please change source to Image.</p>");
					}
				}
			}
		},

		onTiledSlideReady : function(){

			if( SDTiled.isTiled() ){
				var $iframe = $('#slidedeck-preview');
				var $slidedeck = $iframe.contents().find('.slidedeck-tiled');

				if( $slidedeck.length ){
					var slidedeckHeight = $slidedeck.height();

					$iframe.addClass('sdt-iframe');
					$('#slidedeck-slide-dimensions').hide();
					 $iframe.height(slidedeckHeight);
					SDTiled.addImageCSS( $slidedeck );
				}
			}
		},

		//Cycle2's ready event is buggy and cannot be relied upon
		createSliderLoadedEvent :function(){

			var height;
			var $iframe = $(this);
			var $slidedeck = $iframe.contents().find('.slidedeck-tiled').find('.slidedeck-tiled-lens');

			if( ! $slidedeck.length ){
				$iframe.removeClass('.sdt-iframe');
			}
/*
			var interval = setInterval(function(){
				if( $slidedeck.find('.sdcycle-slide').length && $slidedeck.height() > 1 ){
					height = $slidedeck.height();
					setTimeout(function(){
						clearInterval(interval);
					}, 3000 );
					$iframe.trigger( 'tiledSlideisReady', height );
				}
			}, 400 );
*/
		},

	    addImageCSS : function( $slidedeck ){

	    	var layerWidth, slidedeckWidth, widthInPercentage, left, top, zIndex;

	    	$slidedeck.each(function(){

	    		slidedeckWidth = $(window).width();

	    		$(this).find('.sdt-slide-layer').each(function(){

					layerWidth        = $(this).find('img').attr('width');
					widthInPercentage = ( layerWidth / slidedeckWidth ) * 100;
					left              = $(this).data('position-x') > -1 ? $(this).data('position-x') : 20;
					top               = $(this).data('position-y') > -1 ? $(this).data('position-y') : 20;
					zIndex            = $(this).data('z-index') ? $(this).data('z-index') : 20;

	    			$(this).css( {
	    				'width' : widthInPercentage + '%',
	    				'z-index' : zIndex,
	    				'left' : left + '%',
	    				'top' : top + '%'
	    			});

	    		});
	    	});
	    },

	    addLensClasses : function()
	    {
            var titletiled = $(this).find('.title').text().trim();
	    	if( titletiled == 'Tiled' ){
	    	    $('#slidedeck-custom-slide-editor, #slidedeck-form-body').addClass('sdt-slide-editor');
	    	}else{
	    	    $('#slidedeck-custom-slide-editor, #slidedeck-form-body').removeClass('sdt-slide-editor').find('.sdt-slide-editor').removeClass('sdt-slide-editor');
	    	    $('#slidedeck-preview').removeClass('sdt-iframe');
	    	}

	    	SDTiled.removeOtherSlideEditors();
	    },

	    removeOtherSlideEditors : function(){
	    	if( this.isTiled() ){
	    		var backgroundImageSrc;
	    		$('#slidedeck-content-control').find('li.slide').each(function(){
	    			backgroundImageSrc = $(this).css('background-image');
	    			if( backgroundImageSrc.indexOf( '/textonly/' ) >= 0 || backgroundImageSrc.indexOf( '/video/' ) >= 0 || backgroundImageSrc.indexOf( '/html/' ) >= 0 ){
	    				$(this).addClass('is-tiled-image-only');
	    			}
	    			else{
	    				$(this).removeClass('is-tiled-image-only');
	    			}
	    		});
	    	}
	    }

	};

	SDTiled.init();

});
