

jQuery(function($){

	"use strict";

	if( window.Sdt !== undefined ) return;

	window.Sdt = {

		$slidedeck : $('.slidedeck-tiled-lens'),
		options : {},
		$next : $('.slidedeck-tiled .deck-navigation.horizontal.next'),
		$prev : $('.slidedeck-tiled .deck-navigation.horizontal.prev'),

		init : function(){
			if( this.$slidedeck === undefined ) return;
			this.createSlider();
			this.createMouseTiled();
			this.events();
		},

		events : function(){
			this.$slidedeck.on( 'sdcycle-next', this.next );
			this.$slidedeck.on( 'sdcycle-prev', this.prev );
			$(window).on( 'load', this.showSlidedeck );
			$(window).on( 'load', this.addImageCSS );
			$(document).on( 'keyup', this.keyboard );
		},

		showSlidedeck : function(){
			var _this = Sdt;
			$('.slidedeck-tiled.sdt-loading').removeClass('sdt-loading');
			$(window).on('scroll', _this.createScrollTiled );
		},

		addNavMarkup : function( $slide, $el ){
			var imageSrc = $slide.find('.sd2-slide-background').data('fullsrc');
			$el.html( "<span class='sdt-nav-icon'></span><img src='" + imageSrc + "'>" );
		},

		addNextNav : function( next ){
			var $nextSlide = this.$slidedeck.find('.sdcycle-slide-active').next();
			$nextSlide = ! $nextSlide.length ? this.$slidedeck.find('li.slide').eq(1) : $nextSlide;
			this.addNavMarkup( $nextSlide, this.$next );
		},

		addPrevNav : function( currentIndex ){
			var $prevSlide = this.$slidedeck.find('.sdcycle-slide-active').prev();
			if( $prevSlide.hasClass('.sdcycle-sentinel') ){
				$prevSlide = this.$slidedeck.find('li.slide').last();
			}
			this.addNavMarkup( $prevSlide, this.$prev );
		},

		next : function( event, obj ){
			//Sdt.addNextNav();
		},

		prev : function( event, obj ){
			//Sdt.addPrevNav();
		},

		keyboard : function(event){

			var $slider = Sdt.$slidedeck;

			if( ! $slider.length || ! Sdt.options.keys  ) return;

			if (event.keyCode == 37) {
	           $slider.sdcycle('prev');
	        } else if (event.keyCode == 39) {
	           $slider.sdcycle('next');
		    }
		},

		addHoverClass : function(){
			var options = this.getSliderOptions();
			if( options.display_nav_arrows ){
				$('.slidedeck-tiled').addClass( 'sdt-arrow-' + options.display_nav_arrows );
			}
		},

		getSliderOptions : function(){
			var $el;
			this.$slidedeck.find('.slide').each(function(){
				if( $(this).find('ul.sdt-layers').length ){
					$el = $(this).find('ul.sdt-layers');
				}
			});

			if( $el === undefined ) return {};

			return {
				'continueScrolling'  : $el.data('continuescrolling'),
				'autoPlay'           : $el.data('autoplay'),
				'autoPlayInterval'   : $el.data('autoplayinterval'),
				'cycle'              : $el.data('cycle'),
				'slideTransition'    : $el.data('slidetransition'),
				'speed'              : $el.data('speed'),
				'keys'               : $el.data('keys'),
				'touch'              : $el.data('touch'),
				'scroll'             : $el.data('scroll'),
				'start'              : $el.data('start'),
				'display_nav_arrows' : $el.data('display-nav-arrows'),
			};
		},

		createSlider : function(){

			try{
				this.options = this.getSliderOptions();

				var cycleOptions = {
						slides  : '>li',
						next    : '.deck-navigation.horizontal.next',
						prev    : '.deck-navigation.horizontal.prev',
						paused  : this.options.autoPlay ? false : true,
						log     : false,
						fx      : this.options.slideTransition ? this.options.slideTransition : 'tileSlide',
						loop    : this.options.cycle ? 0 : 1, //Loop Playback
						speed   : this.options.speed ? parseInt(this.options.speed) : 1000, //Animation Speed
						timeout : this.options.autoPlayInterval ? parseInt(this.options.autoPlayInterval) * 1000 : 3000, //Autoplay Interval
						swipe 	: this.options.touch ? true : false,
				};

				this.$slidedeck.sdcycle( cycleOptions );

				this.addHoverClass();

			}
			catch(error)
			{
				console.log(error, "Important: Its not an error, its just a catch notice.");
			}

		},

		createMouseTiled : function(  ){
			var $scene, $image, src, tiled;

			var $slidedeck = $('.lens-tiled');

			if( ! $slidedeck.length ) return;

			$('.slidedeck-tiled').addClass('sdt-tiled-active');

			$slidedeck.find('.slide').each(function(){
				if( ! $(this).hasClass('cycle-sentinel') ){
					$scene = $(this).find('.sdt-layers.tiled-type-mousemove');
					tiled = $scene.tiled();
				}
			});
		},

		createScrollTiled : function(){
			var $image, src, extra, scrollTop, $slideImage, topOffset, difference, half;

			var $slidedeck = $('.lens-tiled');

			if( ! $slidedeck.length ) return;

			$('.slidedeck-tiled').addClass('sdt-tiled-active');

			$slidedeck.find('.slide').each(function(){
				if( ! $(this).hasClass('cycle-sentinel') ){

					if( $(this).find('.tiled-type-scroll').length ){

						$image      = $(this).find('img.sd2-slide-background');
						src         = $image.data('fullsrc');
						extra       = 50;
						scrollTop   = $(window).scrollTop();
						$slideImage = $(this).find('img.sd2-slide-background');
						topOffset   = $(this).offset().top - extra;

						if ( topOffset < scrollTop )
						{
							difference = scrollTop  - topOffset;
							half       = (difference / 2) + 'px';
							$slideImage.css('top', half);
						}
						else
						{
							$slideImage.css('top', '0');
						}

					}
				}
			});

		},

		showHideLayers : function( index )
		{
			var $slides = this.$slidedeck.find('.slide').not('.cycle-sentinel');
			var _this = this;

			if( typeof index === 'undefined' ){
				this.fadeIn( $slides.find('.sdt-layers') );
			}
			else{
				var $currentSlide = $slides.eq(index);
				this.fadeOut( $slides.not($currentSlide).find('.sdt-layers') );
				$currentSlide.find('.sdt-layers').hide();
				setTimeout(function(){
					_this.fadeIn( $currentSlide.find('.sdt-layers') );
				}, 1500 );
			}

		},

		fadeIn : function( $el ){
			$el.removeClass('fadeIn-animated fadeIn fadeOut fadeOut-animated');
			$el.show().addClass('fadeIn-animated fadeIn');
		},

		fadeOut : function( $el ){
			$el.removeClass('fadeIn-animated fadeIn fadeOut fadeOut-animated');
			$el.addClass('fadeOut-animated fadeOut');
		},

	    addImageCSS : function(){

	    	var $slidedeck = $('.lens-tiled');
	    	var layerWidth, slidedeckWidth, widthInPercentage, left, top, zIndex;

	    	$slidedeck.each(function(){

	    		slidedeckWidth = $(window).width();

	    		$(this).find('.sdt-slide-layer').each(function(){

					layerWidth        = $(this).find('img').attr('width');
					widthInPercentage = ( layerWidth / slidedeckWidth ) * 100;
					left              = $(this).data('position-x') > -1 ? $(this).data('position-x') : 20;
					top               = $(this).data('position-y') > -1 ? $(this).data('position-y') : 20;
					zIndex            = $(this).data('z-index') ? $(this).data('z-index') : 20;

	    			$(this).css( {
	    				'width' : widthInPercentage + '%',
	    				'z-index' : zIndex,
	    				'left' : left + '%',
	    				'top' : top + '%'
	    			});

	    		});
	    	});

	    },


	};

	window.Sdt.init();

});
